/********************************************************************/
/**                                                                **/
/**   exapmle_1.c                                                  **/
/**                                                                **/
/**   A universal generator for multivariate T-concave             **/
/**   distributions                                                **/
/**                                                                **/
/**   generate normal distributed variates                         **/
/**                                                                **/
/**   domain:  IR^N                                                **/
/**   density: f(x_1,...x_N) = exp(-x_1^2-...-x_N^2)               **/
/**                                                                **/
/**   USE DEFAULT VALUES                                           **/
/**                                                                **/
/**   authors: Josef Leydold                                       **/
/**   last modified: Mon Mar  9 13:38:00 MET 1998                  **/
/**                                                                **/
/********************************************************************/              

/* the header file */
#include "tdrmv.h"

/*------------------------------------------------------------------*/
/* Compiling options in config.h                                    */
/*                                                                  */
/*    N          2, ..., 14                                          */
/*    MODE       0 or 1                                             */
/*    RECTANGLE  0                                                  */
/*    GAMMA      1 or 2                                             */
/*                                                                  */
/*------------------------------------------------------------------*/

#define SAMPLE_SIZE   100      /* sample size                       */

/*------------------------------------------------------------------*/

/* print random point */
static void print_rpoint(double *v, int n);

/*------------------------------------------------------------------*/

/*******************************************************************

  Main

  ******************************************************************/

int main()
{
  C_ROOT *hat;         /* pointer to hat */
  double rpoint[N];    /* array for random point */
  int i;

  /* density */
  double normal();                 /* density function */
  void grad_normal();              /* gradient of density */
  
  /* hat using default values */
  hat = get_hat(get_constr_default(),normal,grad_normal);
  
  /* sample */
  for( i=0; i<SAMPLE_SIZE; i++ ) {
    sample_point(hat,rpoint);
    print_rpoint(rpoint,N);
  }
      
#if DEBUG > 0
  db_close(hat);
#endif
  
  exit(0);
} /* end of main() */

/*-----------------------------------------------------------------*/

static void print_rpoint(double *v, int n)
/*
  print random point
*/
{
  int i;

  printf("( %f",v[0]);
  for( i=1; i<n; i++ )
    printf(", %f",v[i]);
  printf(" )\n");

} /* end of db_print_rpoint() */

/*-----------------------------------------------------------------*/

/*******************************************************************

  standard normal distribution

  ******************************************************************/

/*-----------------------------------------------------------------*/

double normal(double *coord)
/*
  normal density function
*/
{
  int i;             /* aux counter */
  double f=0;       /* density function */

  /* gaussian density */
  for( i=0; i<N; i++ )
    f += coord[i] * coord[i];

  return( exp(-f) );
} /* end of normal() */

/*-----------------------------------------------------------------*/

void grad_normal(double *coord, double *grad)
/*
  gradient of normal density function
*/
{
  int i;              /* aux counter */
  double f=0;             /* density at point */

  /* d(e^{-\sum j*x_j^2})/d x_i = -2*i*x_i*e^{-\sum j*x_j^2} */

  for( i=0; i<N; i++ )
    f += coord[i] * coord[i];
  f = exp( -f );

  for( i=0; i<N; i++ )
    grad[i] = (-2) * coord[i] * f;

} /* end of grad_normal() */

/*-----------------------------------------------------------------*/

